import socket
from header import encode
import getport
import threading



class client:
    def __init__(self, relayServer):
        self.__realyServer = relayServer
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.__sock.bind(('0.0.0.0', getport.get()))
        threading.Thread(target=self.recvfrom).start()
    def sendto(self, data, address):
        data = encode(data, address)
        self.__sock.sendto(data, self.__realyServer)

    def recvfrom(self,size=4096):
            return self.__sock.recvfrom(size)







from udp_relay.header import *

def test_main():
    address = ('192.168.0.2', 1092)
    data = b'asdlfja;lksjfO(*)*#(*$)@#&$)@#(*$LJDSJF'
    encodeData = encode(data, address)
    d, a = decode(encodeData)
    assert a == address
    assert d == data






"""
   extera header for client (8 Bytes)
+------+-------------------------------+
| flag |      Ip address(v4)  | port   |
|2 byte|        4 bytes       | 2 bytes|
+--------------------------------------+
"""

AUTH_FLAG=0x7f7f
# encode extera udp header
def encode(data, address):
    flag = AUTH_FLAG.to_bytes(2, 'big')
    ip = ip2bytes(address[0])
    port = int(address[1]).to_bytes(2, 'big')
    return flag + ip + port + data


# decode udp header
# return (address, data)
# return (0,0) if flag is wrong
def decode(data):
    if int.from_bytes(data[:2], 'big') == AUTH_FLAG:
        ip = byte2IP(data[2:6])
        port = byte2Int(data[6:8])
        data = data[8:]
        return data,(ip,port)
    return 0, 0

def byte2IP(bytes):

    assert len(bytes) == 4
    # ip=''
    # for i in bytes:
    #     ip += str(i)
    # return ip
    return '.'.join(map(str, bytes))

def byte2Int(bytes):
    return int.from_bytes(bytes, 'big')

def ip2bytes(ipaddress):
    ipadd = ipaddress.split('.')
    assert len(ipadd) == 4
    ipadd = list(map(int, ipadd))
    ipNum = int.from_bytes(ipadd, 'big')
    return ipNum.to_bytes(4, 'big')




# A snoob udp relay server
# This file aim to bypass NAT
# run this on server which have public IP
# 2017-12-12

import logging
import socket
import sys
from header import decode
logger = logging.getLogger('relay-server')
# logging.basicConfig(filename='udp_relay_server.log', level='DEBUG')
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)

class server:
    def __init__(self, port=10002):
        self.__sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        self.__sock.bind(('0.0.0.0', port))
        self.__client_address = ()
        logger.info('server start on ' + str(port))

    def start(self):
        while True:
            data, address = self.__sock.recvfrom(4096)
            logger.info('收到消息!')
            if data:
                decode_data, decode_address = decode(data)
                if decode_address != 0:
                    logger.info('转发来自 ' + str(decode_address) + '的流量:' + str(decode_data))
                    self.__sock.sendto(decode_data, decode_address)
                else:
                    if len(self.__client_address) == 0:
                        logger.error('无client!')
                        return
                    else:
                        logger.info(str(address) + ' -> ' + str(self.__client_address) + ' || ' + str(data))
                        self.__sock.sendto(data, self.__client_address)







    def close(self):
        self.__sock.close()

if __name__ == '__main__':
    server = server()
    server.start()



import socket
import sys
import time

PORT = 10000
sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
server_address = ('localhost', PORT)
print('UDP echo server start on port :  ' + str(PORT) + '\n')
sock.bind(server_address)

while True:
    data, address = sock.recvfrom(4096)

    print('[' + str(time.time()) + ']received %s bytes from %s' % (len(data), address))
    print(data)

    if data:
        sent = sock.sendto(data, address)
